package sep.io;

import java.util.*;

/**
 * This class stores and retrieves a command line argument list 
 * or any other array of Strings. 
 * <BR><!--#include file="./source.inc"--><BR>
 * The class distinguishes three types of arguments: <P> 
 *
 * @param -flag   Flags starting with a dash. @see containsKey is a query
 *         for the existence of an argument and is the main method
 *         to querry for flags. <P>
 * 
 * @param par=3   Parameter definitions follow the pattern "string=string".
 *                The parameter value "3" can be extracted with something 
 *                like get("par"). Alternative get methods return alternative 
 *                interpretations of "3", e.g. string, float, integer, 
 *                one-element float array.
 *
 *         @see ArgumentKeeper#get,
 *         @see ArgumentKeeper#getString(), 
 *         @see ArgumentKeeper#getInt(), 
 *         @see ArgumentKeeper#getFloat() 
 *         @see ArgumentKeeper#getIntArray(), 
 *         @see ArgumentKeeper#getFloatArray(). <P> 
 * 
 * @param name    Any other string is assumed to be just a name that the 
 *         ArgumentKeeper should remember in the order they were 
 *         listed. getNames() returns the String array of all these
 *         strings. <P>
 *
 * BUGS: ArgumentKeeper should include ParaKeeper: duplicated code!
 *       ArgumentKeeper and ParaKeeper should recursively read files indicated by 
 *       par=filename.p. 
 *       We need quotes for string arguments so that blanks etc can be used inside 
 *       of strings without being detected by the tokenizer.  
 *
 * @author Matthias Schwab
 */
public class ArgumentKeeper {
  private Hashtable para  = new Hashtable();
  private Vector    names = new Vector();
  
  public ArgumentKeeper(String[] args) {
    for (int i=0; i < args.length; i++) {
      if ((args[i]).startsWith("-")) {
	para.put(args[i],"true");
      }
      else if ((args[i]).indexOf("=") != -1) {
	int positionOfEqual = (args[i]).indexOf("=");
	String paraName  = (args[i]).substring(0,positionOfEqual);
	String paraValue = (args[i]).substring(paraName.length() + 1);
	para.put(paraName,paraValue); 
      }
      else {
	names.addElement(args[i]);
      }
    }
  }
  
  public int size() {
    return para.size();
  }
  public boolean isEmpty() {
    return para.isEmpty();
  }
  public boolean containsKey(String myflag) {
    return para.containsKey(myflag);
  }
  /** 
   * Will return the number of arguments that start with String "root" 
   * and end with a counter:
   * e.g. args contains n1=4 n2=5 n3=7 and root is "n" than the return 
   * value is 3. 
   */
  public int nOfParamWith(String root) {
    int counter = 1; 
    while ( para.containsKey(root + counter)) {
System.err.println("overload: " + root + counter);
      counter++; 
    }
    return --counter; 
  }


  public String          get      (String paraName) {
    return (String) para.get      (      paraName);
  }
  public String          getString(String paraName) {
    return (String)para.get       (       paraName);
  }
  public int     getInt    (   String paraName) {
    return new      Integer(getString(paraName)).intValue    ();
  }
  public float   getFloat  (   String paraName) {
    return new      Float  (getString(paraName)).floatValue  ();
  }
  public double  getDouble (   String paraName) {
    return new      Double (getString(paraName)).doubleValue ();
  }
  public boolean getBoolean(   String paraName) {
    return new      Boolean(getString(paraName)).booleanValue();
  }
  /*
  public boolean getBoolean(String paraName) {
    return  getString(paraName).toLowerCase().equals( "true" );
  }
  */
  public String[] getNames() {
    String[] dummy = new String[names.size()];
    names.copyInto(dummy);
    return dummy;
  }

  // with String defaults 
  public String  getString (String paraName, String deflt) {
    Object val = para.get(paraName);
    if ( val == null ) para.put(paraName, deflt); 
    return getString(paraName);  
  }
  public int     getInt    (String paraName, String deflt) {
    Object val = para.get(paraName);
    if ( val == null ) para.put(paraName, deflt); 
    return getInt(paraName);  
  }
  public float   getFloat  (String paraName, String deflt) {
    Object val = para.get(paraName);
    if ( val == null ) para.put(paraName, deflt); 
    return getFloat(paraName);  
  }
  public double  getDouble (String paraName, String deflt) {
    Object val = para.get(paraName);
    if ( val == null ) para.put(paraName, deflt); 
    return getDouble(paraName);  
  }
  public boolean getBoolean(String paraName, String deflt) {
    Object val = para.get(paraName);
    if ( val == null ) para.put(paraName, deflt); 
    return getBoolean(paraName);  
  }

  // with Type defaults 
  public int     getInt    (String paraName, int deflt) {
    Object val = para.get(paraName);
    if ( val == null ) para.put(paraName, Integer.toString(deflt)); 
    return getInt(paraName);  
  }
  public float   getFloat  (String paraName, float deflt) {
    Object val = para.get(paraName);
    if ( val == null ) para.put(paraName, Float.toString(deflt)); 
    return getFloat(paraName);  
  }
  public double  getDouble (String paraName, double deflt) {
    Object val = para.get(paraName);
    if ( val == null ) para.put(paraName, Double.toString(deflt)); 
    return getDouble(paraName);  
  }
  public boolean getBoolean(String paraName, boolean deflt) {
    Object val = para.get(paraName);
    if ( val == null ) para.put(paraName, new Boolean(deflt).toString()); 
    return getBoolean(paraName);  
  }

  // JOEL's former version: 
  /* 
  public String  getString (String paraName, String deflt) {
    Object val = para.get(paraName);
    return ( val == null ) ? deflt : (String) val;
  }
  public int     getInt    (String paraName, String deflt) {
    return new Integer(  getString(paraName, deflt)).intValue    ();
  }
  public float   getFloat  (String paraName, String deflt) {
    Float    dummy = new Float(0);
    return  (dummy.valueOf(getString(paraName, deflt))).floatValue();
  }
  public double  getDouble (String paraName, String deflt) {
    Double   dummy = new Double(0);
    return  (dummy.valueOf(getString(paraName, deflt))).doubleValue();
  }
  public boolean getBoolean(String paraName, String deflt) {
    return  getString(paraName, deflt).toLowerCase().equals( "true" );
  }
  */

  /** 
   * @param par=1,2,3 returns int i = int[3] with i[0]=1 i[1]=2 i[3]=2
   */
  public int[] getIntArray(String paraName){ 
    String dummy = getString(paraName);
    StringTokenizer tok = new StringTokenizer(dummy, "," );
    int[] retVal = new int[tok.countTokens()]; 
    int k = 0; 
    while (tok.hasMoreTokens()) {
      retVal[k] = Integer.valueOf(tok.nextToken()).intValue();
      k++;
    }
    return retVal;
  }
  /** 
   * alternative default version: the default is not a string, but of 
   * the return type. I think it might be superior. We could offer both. 
   */
  public int[] getIntArray(String paraName, int[] deflt) {
    Object val = para.get(paraName); 
    return ( val == null ) ? deflt : getIntArray(paraName); 
  }
  public int[] getIntArray(String paraName, String deflt) {
    Object val = para.get(paraName);
    if ( val == null ) 	para.put(paraName, deflt); 
    return getIntArray(paraName); 
  }

  /** 
   * @param par=.5,2 returns float i = float[3] with i[0]=.5f i[1]=2.f 
   */
  public float[] getFloatArray(String paraName){ 
    String dummy = getString(paraName);
    StringTokenizer tok = new StringTokenizer(dummy, "," );
    float[] retVal = new float[tok.countTokens()]; 
    int k = 0; 
    while (tok.hasMoreTokens()) {
      retVal[k] = Float.valueOf(tok.nextToken()).floatValue();
      k++;
    }
    return retVal;
  }
  /** 
   * alternative default version: the default is not a string, but of 
   * the return type. I think it might be superior. We could offer both. 
   */
  public float[] getFloatArray(String paraName, float[] deflt) {
    Object val = para.get(paraName); 
    return ( val == null ) ? deflt : getFloatArray(paraName); 
  }
  public float[] getFloatArray(String paraName, String deflt) {
    Object val = para.get(paraName);
    if ( val == null ) 	para.put(paraName, deflt); 
    return getFloatArray(paraName); 
  }

  /** 
   * @param par=high,low returns String[] i = String[2] with i[0]=high i[1]=low
   */
  public String[] getStringArray(String paraName){ 
    String dummy = getString(paraName);
    StringTokenizer tok = new StringTokenizer(dummy, "," );
    String[] retVal = new String[tok.countTokens()]; 
    int k = 0; 
    while (tok.hasMoreTokens()) {
      retVal[k] = tok.nextToken();
      k++;
    }
    return retVal;
  }
  /** 
   * alternative default version: the default is not a string, but of 
   * the return type. I think it might be superior. We could offer both. 
   */
  public String[] getStringArray(String paraName, String[] deflt) {
    Object val = para.get(paraName); 
    return ( val == null ) ? deflt : getStringArray(paraName); 
  }
  public String[] getStringArray(String paraName, String deflt) {
    Object val = para.get(paraName);
    if ( val == null ) 	para.put(paraName, deflt); 
    return getStringArray(paraName); 
  }
  /** 
   * <pre>axis1=12,.004f,0.0f,time,sec</pre> 
   */ 
  public rsf.vector.Axis getAxis(String paraName) {
    String dummy = getString(paraName); 
    StringTokenizer tok = new StringTokenizer(dummy, "," );
    if (tok.countTokens() != 5) System.err.println("Error: incomplete Axis.");
    return new rsf.vector.Axis(Integer.valueOf(tok.nextToken()).intValue(), 
			         Float.valueOf(tok.nextToken()).floatValue(),
			         Float.valueOf(tok.nextToken()).floatValue(),
			                       tok.nextToken(), 
			                       tok.nextToken()
			       );
  }
  public rsf.vector.Axis getAxis(String paraName, String deflt) {
    Object val = para.get(paraName); 
    if (val == null ) para.put(paraName, deflt); 
    return getAxis(paraName);
  }
}

