package  sep.operator;

import 	 jam.vector.*;
import 	 jam.operator.*;
import juice.operator.ZeroBiLinOp;

/**
 * This class implements a lot of often duplicated code.<BR>
 * <BR>
 * TYPICAL THINGS TO IMPLMENT WHEN SUBCLASSING:<BR>
 * <UL>
 * <LI>apply()
 * <LI>constructors
 * <LI>copy()
 * <LI>equals()
 * <LI>toString()
 * </UL>
 * The adjoint is created with a clone, so IF YOU NEED SOMETHING TO BE A
 * DEEP COPY, YOU SHOULD OVERRIDE createAdjoint().
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author    Joel Schroeder
 */
public abstract class SepLinearOperator
implements hasAdjoint, hasHessian
{
  /** domain of the forward operator (and range of the adjoint) */
  protected Space           input;
  /** range  of the forward operator (and domain of the forward) */
  protected Space           output;
  /** true/false as the operator is the forward/adjoint */
  protected boolean         isFwd = true;
  /**
   * Pointer to the (relative) adjoint of this operator:<BR>
   * this.isFwd == true  ---> adj.isFwd == false<BR>
   * this.isFwd == false ---> adj.isFwd == true
   */
  protected hasAdjoint              adj  = null;
  protected BiLinearOperatorAdjoint hess = null;

  public SepLinearOperator() {}

  public SepLinearOperator(Space fwdDomain, Space fwdRange) {
    setInput  (fwdDomain);
    setOutput (fwdRange );
  }

  public SepLinearOperator(Space fwdDomain, Space fwdRange, boolean isFwd) {
    this(fwdDomain, fwdRange);
    setForward(isFwd);
  }

  public SepLinearOperator(Space domainAndRange) {
    this(domainAndRange, domainAndRange);
  }

  public SepLinearOperator(Space domainAndRange, boolean isFwd) {
    this(domainAndRange, domainAndRange);
    setForward(isFwd);
  }

  /** copy constructor (shallow copy) */
  public SepLinearOperator(SepLinearOperator src) {
    this.input  = src.input;
    this.output = src.output;
    this.isFwd  = src.isFwd;
  }

  /**
   * Apply the operator:<BR><BR>
   *
   * Depending on if the operator is the forward or adjoint:<BR>
   * forward: out = A(in)<BR>
   * adjoint: in  = A(out)<BR>
   *
   * <BR>
   * CONTRAST THIS INTERFACE WITH THAT OF SepSelfAdjointOperator.apply()
   * <BR>
   *
   * @param add Performs y += Ax instead of y = Ax if true (where {x,y} = {in,out} -- respectively if and only if isFwd == true)
   * @param in  Vector in the domain of the forward (altered if isFwd == false)
   * @param out Vector in the range  of the forward (altered if isFwd == true)
   */
  protected abstract void  apply     (boolean add, Vector in, Vector out);

  protected void  setInput  (Space input   ) { this.input  = input ; }
  protected void  setOutput (Space output  ) { this.output = output; }
  protected void  setForward(boolean isFwd ) { this.isFwd  = isFwd;  }
  protected void  setAdjoint(hasAdjoint adj) { this.adj    = adj;    }

  protected void  createAdjoint() {
    try {
      adj = (hasAdjoint) this.clone();
    }
    catch (Exception e) {
      System.err.println(e);
    }
    ((SepLinearOperator) adj).setForward(!this.isFwd);
    ((SepLinearOperator) adj).setAdjoint(this);
  }

  public boolean equals(SepLinearOperator slo2) {
    return (this.input .equals(slo2.input ) &&
	    this.output.equals(slo2.output) &&
	    this.isFwd == slo2.isFwd);
  }

  // ----------------------------------------------------------------------
  // ---------- The remaining methods are required by the superclass ------
  // ----------------------------------------------------------------------

  public Space   getDomain () { return isFwd ? input:output; }
  public Space   getRange  () { return isFwd ? output:input; }

  public Vector  image(Vector d) {
    Vector r = getRange().newMember();
    image(d, r);
    return r; }

  public void    image(Vector d, Vector r) {
    if (isFwd) apply(false, d, r);
    else       apply(false, r, d);
    }
  
  public void addImage(Vector d, Vector r) {
    if (isFwd) apply(true , d, r);
    else       apply(true , r, d); }

  public void residual(Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d, r); }

  public hasAdjoint     adjoint() {
    if (adj == null) createAdjoint();
    return adj;
  }

  public LinearOperator           getGradientAt  (Vector x) {
    return adjoint();
  }
  public hasAdjoint               getDerivativeAt(Vector x) {
    return this;
  }
  public BiLinearOperatorAdjoint  getHessianAt   (Vector x) {
    if (hess == null) createHessian();
    return hess;
  }

  protected void createHessian() {
    hess = new ZeroBiLinOp(getDomain(), getDomain(), getRange());
  }

  protected void setOutputNull(boolean isFwd, boolean add, Vector d, Vector r){
    if (! add) // zero output! 
      if   (isFwd) r.zero(); 
      else         d.zero();
  }
}
